@@ -1,6 +1,6 @@
 """Expression type checker. This file is conceptually part of TypeChecker."""
 
-from typing import Undefined, cast, List, Tuple, Dict, Function
+from typing import Undefined, cast, List, Tuple, Dict, Function, Union
 
 from mypy.types import (
     Type, AnyType, Callable, Overloaded, NoneTyp, Void, TypeVarDef,
@@ -12,7 +12,8 @@ from mypy.nodes import (
     OpExpr, UnaryExpr, IndexExpr, CastExpr, TypeApplication, ListExpr,
     TupleExpr, DictExpr, FuncExpr, SuperExpr, ParenExpr, SliceExpr, Context,
     ListComprehension, GeneratorExpr, SetExpr, MypyFile, Decorator,
-    UndefinedExpr, ConditionalExpr, ComparisonExpr, TempNode, LITERAL_TYPE
+    UndefinedExpr, ConditionalExpr, ComparisonExpr, TempNode, SetComprehension,
+    DictionaryComprehension, ComplexExpr, LITERAL_TYPE
 )
 from mypy.errors import Errors
 from mypy.nodes import function_type, method_type
@@ -26,7 +27,7 @@ from mypy import messages
 from mypy.infer import infer_type_arguments, infer_function_type_arguments
 from mypy import join
 from mypy.expandtype import expand_type, expand_caller_var_args
-from mypy.subtypes import is_subtype
+from mypy.subtypes import is_subtype, is_more_precise
 from mypy import applytype
 from mypy import erasetype
 from mypy.checkmember import analyse_member_access, type_object_type
@@ -365,12 +366,12 @@ class ExpressionChecker:
             context: Context) -> Tuple[Callable, List[Type]]:
         """Perform second pass of generic function type argument inference.
 
-        The second pass is needed for arguments with types such as func<s(t)>,
-        where both s and t are type variables, when the actual argument is a
-        lambda with inferred types.  The idea is to infer the type variable t
+        The second pass is needed for arguments with types such as Function[[T], S],
+        where both T and S are type variables, when the actual argument is a
+        lambda with inferred types.  The idea is to infer the type variable T
         in the first pass (based on the types of other arguments).  This lets
         us infer the argument and return type of the lambda expression and
-        thus also the type variable s in this second pass.
+        thus also the type variable S in this second pass.
 
         Return (the callee with type vars applied, inferred actual arg types).
         """
@@ -602,15 +603,17 @@ class ExpressionChecker:
         # Fixed function arguments.
         func_fixed = callee.max_fixed_args()
         for i in range(min(len(arg_types), func_fixed)):
-            if not is_subtype(self.erase(arg_types[i]),
-                              self.erase(
-                                  callee.arg_types[i])):
+            # Use is_more_precise rather than is_subtype because it ignores ducktype
+            # declarations. This is important since ducktype declarations are ignored
+            # when performing runtime type checking.
+            if not is_compatible_overload_arg(arg_types[i], callee.arg_types[i]):
                 return False
         # Function varargs.
         if callee.is_var_arg:
             for i in range(func_fixed, len(arg_types)):
-                if not is_subtype(self.erase(arg_types[i]),
-                                  self.erase(callee.arg_types[func_fixed])):
+                # See above for why we use is_more_precise.
+                if not is_compatible_overload_arg(arg_types[i],
+                                                  callee.arg_types[func_fixed]):
                     return False
         return True
 
@@ -701,6 +704,10 @@ class ExpressionChecker:
         """Type check a float literal (trivial)."""
         return self.named_type('builtins.float')
 
+    def visit_complex_expr(self, e: ComplexExpr) -> Type:
+        """Type check a complex literal."""
+        return self.named_type('builtins.complex')
+
     def visit_op_expr(self, e: OpExpr) -> Type:
         """Type check a binary operator expression."""
         if e.op == 'and' or e.op == 'or':
@@ -1143,6 +1150,10 @@ class ExpressionChecker:
         return self.check_generator_or_comprehension(
             e.generator, 'builtins.list', '<list-comprehension>')
 
+    def visit_set_comprehension(self, e: SetComprehension) -> Type:
+        return self.check_generator_or_comprehension(
+            e.generator, 'builtins.set', '<set-comprehension>')
+
     def visit_generator_expr(self, e: GeneratorExpr) -> Type:
         return self.check_generator_or_comprehension(e, 'typing.Iterator',
                                                      '<generator>')
@@ -1151,15 +1162,7 @@ class ExpressionChecker:
                                          type_name: str,
                                          id_for_messages: str) -> Type:
         """Type check a generator expression or a list comprehension."""
-
-        self.chk.binder.push_frame()
-        for index, sequence, conditions in zip(gen.indices, gen.sequences,
-                                               gen.condlists):
-            sequence_type = self.chk.analyse_iterable_item_type(sequence)
-            self.chk.analyse_index_variables(index, sequence_type, gen)
-            for condition in conditions:
-                self.accept(condition)
-        self.chk.binder.pop_frame()
+        self.check_for_comp(gen)
 
         # Infer the type of the list comprehension by using a synthetic generic
         # callable type.
@@ -1174,6 +1177,38 @@ class ExpressionChecker:
         return self.check_call(constructor,
                                [gen.left_expr], [nodes.ARG_POS], gen)[0]
 
+    def visit_dictionary_comprehension(self, e: DictionaryComprehension):
+        """Type check a dictionary comprehension."""
+        self.check_for_comp(e)
+
+        # Infer the type of the list comprehension by using a synthetic generic
+        # callable type.
+        key_tv = TypeVar('KT', -1, [], self.chk.object_type())
+        value_tv = TypeVar('VT', -2, [], self.chk.object_type())
+        constructor = Callable([key_tv, value_tv],
+                               [nodes.ARG_POS, nodes.ARG_POS],
+                               [None, None],
+                               self.chk.named_generic_type('builtins.dict', [key_tv, value_tv]),
+                               self.chk.named_type('builtins.function'),
+                               '<dictionary-comprehension>',
+                               [TypeVarDef('KT', -1, None, self.chk.object_type()),
+                                TypeVarDef('VT', -2, None, self.chk.object_type())])
+        return self.check_call(constructor,
+                               [e.key, e.value], [nodes.ARG_POS, nodes.ARG_POS], e)[0]
+
+    def check_for_comp(self, e: Union[GeneratorExpr, DictionaryComprehension]) -> None:
+        """Check the for_comp part of comprehensions. That is the part from 'for':
+        ... for x in y if z
+        """
+        self.chk.binder.push_frame()
+        for index, sequence, conditions in zip(e.indices, e.sequences,
+                                               e.condlists):
+            sequence_type = self.chk.analyse_iterable_item_type(sequence)
+            self.chk.analyse_index_variables(index, sequence_type, e)
+            for condition in conditions:
+                self.accept(condition)
+        self.chk.binder.pop_frame()
+
     def visit_undefined_expr(self, e: UndefinedExpr) -> Type:
         return e.type
 
@@ -1246,10 +1281,6 @@ class ExpressionChecker:
             r.append(self.strip_parens(n))
         return r
 
-    def erase(self, type: Type) -> Type:
-        """Replace type variable types in type with Any."""
-        return erasetype.erase_type(type)
-
 
 def is_valid_argc(nargs: int, is_var_arg: bool, callable: Callable) -> bool:
     """Return a boolean indicating whether a call expression has a
@@ -1397,3 +1428,30 @@ class HasErasedComponentsQuery(types.TypeQuery):
 
     def visit_erased_type(self, t: ErasedType) -> bool:
         return True
+
+
+def is_compatible_overload_arg(actual: Type, formal: Type) -> bool:
+    if (isinstance(actual, NoneTyp) or isinstance(actual, AnyType) or
+            isinstance(formal, AnyType) or isinstance(formal, TypeVar) or
+            isinstance(formal, Callable)):
+        # These could match anything at runtime.
+        return True
+    if isinstance(actual, UnionType):
+        return any(is_compatible_overload_arg(item, formal)
+                   for item in actual.items)
+    if isinstance(formal, UnionType):
+        return any(is_compatible_overload_arg(actual, item)
+                   for item in formal.items)
+    if isinstance(formal, Instance):
+        if isinstance(actual, Callable):
+            actual = actual.fallback
+        if isinstance(actual, Overloaded):
+            actual = actual.items()[0].fallback
+        if isinstance(actual, TupleType):
+            actual = actual.fallback
+        if isinstance(actual, Instance):
+            return formal.type in actual.type.mro
+        else:
+            return False
+    # Fall back to a conservative equality check for the remaining kinds of type.
+    return is_same_type(erasetype.erase_type(actual), erasetype.erase_type(formal))
