@@ -9,7 +9,7 @@ This module can be run as a script (lex.py FILE).
 import re
 
 from mypy.util import short_type
-from typing import List, Undefined, Function, Dict, Any, Match, Pattern
+from typing import List, Undefined, Function, Dict, Any, Match, Pattern, Set
 
 
 class Token:
@@ -105,6 +105,10 @@ class FloatLit(Token):
     """Float literal"""
 
 
+class ComplexLit(Token):
+    """Complex literal"""
+
+
 class Punct(Token):
     """Punctuator (e.g. comma, '(' or '=')"""
 
@@ -148,23 +152,31 @@ STR_CONTEXT = 1
 COMMENT_CONTEXT = 2
 
 
-def lex(string: str, first_line: int = 1) -> List[Token]:
+def lex(string: str, first_line: int = 1, pyversion: int = 3) -> List[Token]:
     """Analyze string and return an array of token objects.
 
     The last token is always Eof.
     """
-    l = Lexer()
+    l = Lexer(pyversion)
     l.lex(string, first_line)
     return l.tok
 
 
 # Reserved words (not including operators)
-keywords = set([
+keywords_common = set([
     'as', 'assert', 'break', 'class', 'continue', 'def', 'del', 'elif',
     'else', 'except', 'finally', 'from', 'for', 'global', 'if', 'import',
     'lambda', 'pass', 'raise', 'return', 'try', 'while', 'with',
     'yield'])
 
+# Reserved words specific for Python version 2
+# TODO (jukka): 'print' should be here, but it breaks the parsing of Python 2
+#               builtins, since they also define the function 'print'.
+keywords2 = set([]) # type: Set[str]
+
+# Reserved words specific for Python version 3
+keywords3 = set(['nonlocal'])
+
 # Alphabetical operators (reserved words)
 alpha_operators = set(['in', 'is', 'not', 'and', 'or'])
 
@@ -279,7 +291,7 @@ class Lexer:
     # newlines within parentheses/brackets.
     open_brackets = Undefined(List[str])
 
-    def __init__(self) -> None:
+    def __init__(self, pyversion: int = 3) -> None:
         self.map = [self.unknown_character] * 256
         self.tok = []
         self.indents = [0]
@@ -302,6 +314,10 @@ class Lexer:
                             ('-+*/<>%&|^~=!,@', self.lex_misc)]:
             for c in seq:
                 self.map[ord(c)] = method
+        if pyversion == 2:
+            self.keywords = keywords_common | keywords2
+        if pyversion == 3:
+            self.keywords = keywords_common | keywords3
 
     def lex(self, s: str, first_line: int) -> None:
         """Lexically analyze a string, storing the tokens at the tok list."""
@@ -362,11 +378,14 @@ class Lexer:
 
     # Regexps used by lex_number
 
-    # Decimal/hex/octal literal
+    # Decimal/hex/octal literal or integer complex literal
     number_exp1 = re.compile('0[xXoO][0-9a-fA-F]+|[0-9]+')
-    # Float literal, e.g. '1.23' or '12e+34'
+    # Float literal, e.g. '1.23' or '12e+34' or '1.2j'
     number_exp2 = re.compile(
         r'[0-9]*\.[0-9]*([eE][-+]?[0-9]+)?|[0-9]+[eE][-+]?[0-9]+')
+    # Complex literal, e.g. '3j' or '1.5e2J'
+    number_complex = re.compile(
+        r'([0-9]*\.[0-9]*([eE][-+]?[0-9]+)?|[0-9]+([eE][-+]?[0-9]+)?)[jJ]')
     # These characters must not appear after a number literal.
     name_char_exp = re.compile('[a-zA-Z0-9_]')
 
@@ -377,20 +396,24 @@ class Lexer:
         """
         s1 = self.match(self.number_exp1)
         s2 = self.match(self.number_exp2)
+        sc = self.match(self.number_complex)
 
-        maxlen = max(len(s1), len(s2))
+        maxlen = max(len(s1), len(s2), len(sc))
         if self.name_char_exp.match(
                 self.s[self.i + maxlen:self.i + maxlen + 1]) is not None:
             # Error: alphanumeric character after number literal.
             s3 = self.match(re.compile('[0-9][0-9a-zA-Z_]*'))
             maxlen = max(maxlen, len(s3))
             self.add_token(LexError(' ' * maxlen, NUMERIC_LITERAL_ERROR))
-        elif len(s1) > len(s2):
+        elif len(s1) == maxlen:
             # Integer literal.
             self.add_token(IntLit(s1))
-        else:
+        elif len(s2) == maxlen:
             # Float literal.
             self.add_token(FloatLit(s2))
+        else:
+            # Complex literal.
+            self.add_token(ComplexLit(sc))
 
     name_exp = re.compile('[a-zA-Z_][a-zA-Z0-9_]*')
 
@@ -401,7 +424,7 @@ class Lexer:
         Also deal with prefixed string literals such as r'...'.
         """
         s = self.match(self.name_exp)
-        if s in keywords:
+        if s in self.keywords:
             self.add_token(Keyword(s))
         elif s in alpha_operators:
             self.add_token(Op(s))
