@@ -32,9 +32,12 @@ INVALID_EXCEPTION = 'Exception must be derived from BaseException'
 INVALID_EXCEPTION_TYPE = 'Exception type must be derived from BaseException'
 INVALID_RETURN_TYPE_FOR_YIELD = \
     'Iterator function return type expected for "yield"'
+INVALID_RETURN_TYPE_FOR_YIELD_FROM = \
+    'Iterable function return type expected for "yield from"'
 INCOMPATIBLE_TYPES = 'Incompatible types'
 INCOMPATIBLE_TYPES_IN_ASSIGNMENT = 'Incompatible types in assignment'
 INCOMPATIBLE_TYPES_IN_YIELD = 'Incompatible types in yield'
+INCOMPATIBLE_TYPES_IN_YIELD_FROM = 'Incompatible types in "yield from"'
 INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION = 'Incompatible types in string interpolation'
 INIT_MUST_NOT_HAVE_RETURN_TYPE = 'Cannot define return type for "__init__"'
 GETTER_TYPE_INCOMPATIBLE_WITH_SETTER = \
@@ -115,7 +118,7 @@ class MessageBuilder:
         if self.disable_count <= 0:
             self.errors.report(context.get_line(), msg.strip())
 
-    def format(self, typ: Type) -> str:
+    def format(self, typ: Type, verbose: bool = False) -> str:
         """Convert a type to a relatively short string that is
         suitable for error messages. Mostly behave like format_simple
         below, but never return an empty string.
@@ -130,16 +133,21 @@ class MessageBuilder:
                 # The type of a type object type can be derived from the
                 # return type (this always works).
                 itype = cast(Instance, func.items()[0].ret_type)
-                return self.format(itype)
+                result = self.format(itype)
+                if verbose:
+                    # In some contexts we want to be explicit about the distinction
+                    # between type X and the type of type object X.
+                    result += ' (type object)'
+                return result
             elif isinstance(func, Callable):
-                arg_types = map(self.format, func.arg_types)
-                return_type = self.format(func.ret_type)
-                return 'Function[[{}] -> {}]'.format(", ".join(arg_types), return_type)
+                arg_types = [strip_quotes(self.format(t)) for t in func.arg_types]
+                return_type = strip_quotes(self.format(func.ret_type))
+                return 'Function[[{}], {}]'.format(", ".join(arg_types), return_type)
             else:
                 # Use a simple representation for function types; proper
                 # function types may result in long and difficult-to-read
                 # error messages.
-                return 'functionlike'
+                return 'overloaded function'
         else:
             # Default case; we simply have to return something meaningful here.
             return 'object'
@@ -187,6 +195,10 @@ class MessageBuilder:
             # This is similar to non-generic instance types.
             return '"{}"'.format((cast(TypeVar, typ)).name)
         elif isinstance(typ, TupleType):
+            fallback = self.format_simple(typ.fallback)
+            if fallback != '"tuple"':
+                # Prefer the name of the fallback class (if not tuple), as it's more informative.
+                return fallback
             items = []
             for t in (cast(TupleType, typ)).items:
                 items.append(strip_quotes(self.format(t)))
@@ -203,7 +215,7 @@ class MessageBuilder:
             if len(s) < 40:
                 return s
             else:
-                return 'union(length {})'.format(len(items))
+                return 'union type ({} items)'.format(len(items))
         elif isinstance(typ, Void):
             return 'None'
         elif isinstance(typ, NoneTyp):
@@ -367,6 +379,14 @@ class MessageBuilder:
         elif callee.name == '<list-comprehension>':
             msg = 'List comprehension has incompatible type List[{}]'.format(
                 strip_quotes(self.format(arg_type)))
+        elif callee.name == '<set-comprehension>':
+            msg = 'Set comprehension has incompatible type Set[{}]'.format(
+                strip_quotes(self.format(arg_type)))
+        elif callee.name == '<dictionary-comprehension>':
+            msg = '{} expression in dictionary comprehension has incompatible type {}; expected type {}'.format(
+                'Key' if n == 1 else 'Value',
+                self.format(arg_type),
+                self.format(callee.arg_types[n-1]))
         elif callee.name == '<generator>':
             msg = 'Generator has incompatible item type {}'.format(
                 self.format_simple(arg_type))
@@ -671,6 +691,11 @@ class MessageBuilder:
         self.fail('Signatures of "{}" and "{}" are incompatible'.format(
             method, other_method), context)
 
+    def yield_from_invalid_operand_type(self, expr: Type, context: Context) -> Type:
+        text = self.format(expr) if self.format(expr) != 'object' else expr
+        self.fail('"yield from" can\'t be applied to {}'.format(text), context)
+        return AnyType()
+
 
 def capitalize(s: str) -> str:
     """Capitalize the first character of a string."""
