@@ -6,12 +6,12 @@ representing a source file. Performs only minimal semantic checks.
 
 import re
 
-from typing import Undefined, List, Tuple, Any, Set, cast
+from typing import Undefined, List, Tuple, Any, Set, cast, Union
 
 from mypy import lex
 from mypy.lex import (
     Token, Eof, Bom, Break, Name, Colon, Dedent, IntLit, StrLit, BytesLit,
-    UnicodeLit, FloatLit, Op, Indent, Keyword, Punct, LexError
+    UnicodeLit, FloatLit, Op, Indent, Keyword, Punct, LexError, ComplexLit
 )
 import mypy.types
 from mypy.nodes import (
@@ -23,7 +23,9 @@ from mypy.nodes import (
     TupleExpr, GeneratorExpr, ListComprehension, ListExpr, ConditionalExpr,
     DictExpr, SetExpr, NameExpr, IntExpr, StrExpr, BytesExpr, UnicodeExpr,
     FloatExpr, CallExpr, SuperExpr, MemberExpr, IndexExpr, SliceExpr, OpExpr,
-    UnaryExpr, FuncExpr, TypeApplication, PrintStmt, ImportBase, ComparisonExpr
+    UnaryExpr, FuncExpr, TypeApplication, PrintStmt, ImportBase, ComparisonExpr,
+    StarExpr, YieldFromStmt, YieldFromExpr, NonlocalDecl, DictionaryComprehension,
+    SetComprehension, ComplexExpr
 )
 from mypy import nodes
 from mypy import noderepr
@@ -44,7 +46,7 @@ precedence = {
     '&': 10,
     '^': 9,
     '|': 8,
-    '==': 7, '!=': 7, '<': 7, '>': 7, '<=': 7, '>=': 7, 'is': 7, 'in': 7,
+    '==': 7, '!=': 7, '<': 7, '>': 7, '<=': 7, '>=': 7, 'is': 7, 'in': 7, '*u': 7, # unary * for star expressions
     'not': 6,
     'and': 5,
     'or': 4,
@@ -107,7 +109,7 @@ class Parser:
             self.errors.set_file('<input>')
 
     def parse(self, s: str) -> MypyFile:
-        self.tok = lex.lex(s)
+        self.tok = lex.lex(s, pyversion=self.pyversion)
         self.ind = 0
         self.imports = []
         self.future_options = []
@@ -169,17 +171,35 @@ class Parser:
 
     def parse_import_from(self) -> Node:
         from_tok = self.expect('from')
-        name, components = self.parse_qualified_name()
+
+        # Build the list of beginning relative tokens.
+        relative = 0
+        rel_toks = List[Token]()
+        while self.current_str() == ".":
+            rel_toks.append(self.expect('.'))
+            relative += 1
+
+        # Parse qualified name to actually import from.
+        if self.current_str() == "import":
+            # Empty/defualt values.
+            name = ""
+            components = List[Token]()
+        else:
+            name, components = self.parse_qualified_name()
+
         if name == self.custom_typing_module:
             name = 'typing'
+
+        # Parse import list
         import_tok = self.expect('import')
         name_toks = List[Tuple[List[Token], Token]]()
         lparen = none
         rparen = none
         node = None  # type: ImportBase
         if self.current_str() == '*':
+            # An import all from a module node:
             name_toks.append(([self.skip()], none))
-            node = ImportAll(name)
+            node = ImportAll(name, relative)
         else:
             is_paren = self.current_str() == '('
             if is_paren:
@@ -205,12 +225,12 @@ class Parser:
             if is_paren:
                 rparen = self.expect(')')
             if node is None:
-                node = ImportFrom(name, targets)
+                node = ImportFrom(name, relative, targets)
         br = self.expect_break()
         self.imports.append(node)
         # TODO: Fix representation if there is a custom typing module import.
         self.set_repr(node, noderepr.ImportFromRepr(
-            from_tok, components, import_tok, lparen, name_toks, rparen, br))
+            from_tok, rel_toks, components, import_tok, lparen, name_toks, rparen, br))
         if name == '__future__':
             self.future_options.extend(target[0] for target in targets)
         return node
@@ -416,6 +436,7 @@ class Parser:
           (name token, representation of arguments)
         """
         name_tok = none
+        name = ''
 
         try:
             name_tok = self.expect_type(Name)
@@ -677,6 +698,8 @@ class Parser:
             stmt = self.parse_class_def()
         elif ts == 'global':
             stmt = self.parse_global_decl()
+        elif ts == 'nonlocal' and self.pyversion >= 3:
+            stmt = self.parse_nonlocal_decl()
         elif ts == 'assert':
             stmt = self.parse_assert_stmt()
         elif ts == 'yield':
@@ -697,7 +720,7 @@ class Parser:
         return stmt
 
     def parse_expression_or_assignment(self) -> Node:
-        e = self.parse_expression()
+        e = self.parse_expression(star_expr_allowed=True)
         if self.current_str() == '=':
             return self.parse_assignment(e)
         elif self.current_str() in op_assign:
@@ -726,11 +749,11 @@ class Parser:
         assigns = [self.expect('=')]
         lvalues = [lv]
 
-        e = self.parse_expression()
+        e = self.parse_expression(star_expr_allowed=True)
         while self.current_str() == '=':
             lvalues.append(e)
             assigns.append(self.skip())
-            e = self.parse_expression()
+            e = self.parse_expression(star_expr_allowed=True)
         br = self.expect_break()
 
         type = self.parse_type_comment(br, signature=False)
@@ -743,6 +766,8 @@ class Parser:
         expr = None  # type: Node
         if not isinstance(self.current(), Break):
             expr = self.parse_expression()
+            if isinstance(expr, YieldFromExpr): # "yield from" expressions can't be returned.
+                return None
         br = self.expect_break()
         node = ReturnStmt(expr)
         self.set_repr(node, noderepr.SimpleStmtRepr(return_tok, br))
@@ -771,16 +796,41 @@ class Parser:
         self.set_repr(node, noderepr.SimpleStmtRepr(assert_tok, br))
         return node
 
-    def parse_yield_stmt(self) -> YieldStmt:
+    def parse_yield_stmt(self) -> Union[YieldStmt, YieldFromStmt]:
         yield_tok = self.expect('yield')
         expr = None  # type: Node
-        if not isinstance(self.current(), Break):
-            expr = self.parse_expression()
-        br = self.expect_break()
         node = YieldStmt(expr)
+        if not isinstance(self.current(), Break):
+            if isinstance(self.current(), Keyword) and self.current_str() == "from":  # Not go if it's not from
+                from_tok = self.expect("from")
+                expr = self.parse_expression()  # Here comes when yield from is not assigned
+                node_from = YieldFromStmt(expr)
+                br = self.expect_break()
+                self.set_repr(node_from, noderepr.SimpleStmtRepr(yield_tok, br))
+                return node_from  # return here, we've gotted the type
+            else:
+                expr = self.parse_expression()
+                node = YieldStmt(expr)
+        br = self.expect_break()
         self.set_repr(node, noderepr.SimpleStmtRepr(yield_tok, br))
         return node
 
+    def parse_yield_from_expr(self) -> YieldFromExpr:
+        y_tok = self.expect("yield")
+        expr = None # type: Node
+        node = YieldFromExpr(expr)
+        if self.current_str() == "from":
+            f_tok = self.expect("from")
+            tok = self.parse_expression()  # Here comes when yield from is assigned to a variable
+            node = YieldFromExpr(tok)
+        else:
+            # TODO
+            # Here comes the yield expression (ex:  x = yield 3 )
+            # tok = self.parse_expression()
+            # node = YieldExpr(tok)  # Doesn't exist now
+            pass
+        return node
+
     def parse_del_stmt(self) -> DelStmt:
         del_tok = self.expect('del')
         expr = self.parse_expression()
@@ -812,6 +862,23 @@ class Parser:
 
     def parse_global_decl(self) -> GlobalDecl:
         global_tok = self.expect('global')
+        name_toks, names, commas = self.parse_identifier_list()
+        br = self.expect_break()
+        node = GlobalDecl(names)
+        self.set_repr(node, noderepr.GlobalDeclRepr(global_tok, name_toks,
+                                                    commas, br))
+        return node
+
+    def parse_nonlocal_decl(self) -> NonlocalDecl:
+        nonlocal_tok = self.expect('nonlocal')
+        name_toks, names, commas = self.parse_identifier_list()
+        br = self.expect_break()
+        node = NonlocalDecl(names)
+        self.set_repr(node, noderepr.NonlocalDeclRepr(nonlocal_tok, name_toks,
+                                                      commas, br))
+        return node
+
+    def parse_identifier_list(self) -> Tuple[List[Token], List[str], List[Token]]:
         names = List[str]()
         name_toks = List[Token]()
         commas = List[Token]()
@@ -822,11 +889,7 @@ class Parser:
             if self.current_str() != ',':
                 break
             commas.append(self.skip())
-        br = self.expect_break()
-        node = GlobalDecl(names)
-        self.set_repr(node, noderepr.GlobalDeclRepr(global_tok, name_toks,
-                                                    commas, br))
-        return node
+        return name_toks, names, commas
 
     def parse_while_stmt(self) -> WhileStmt:
         is_error = False
@@ -851,7 +914,7 @@ class Parser:
 
     def parse_for_stmt(self) -> ForStmt:
         for_tok = self.expect('for')
-        index, commas = self.parse_for_index_variables()
+        index = self.parse_for_index_variables()
         in_tok = self.expect('in')
         expr = self.parse_expression()
 
@@ -865,31 +928,30 @@ class Parser:
             else_tok = none
 
         node = ForStmt(index, expr, body, else_body)
-        self.set_repr(node, noderepr.ForStmtRepr(for_tok, commas, in_tok,
-                                                 else_tok))
+        self.set_repr(node, noderepr.ForStmtRepr(for_tok, in_tok, else_tok))
         return node
 
-    def parse_for_index_variables(self) -> Tuple[List[Node], List[Token]]:
+    def parse_for_index_variables(self) -> Node:
         # Parse index variables of a 'for' statement.
-        index = List[Node]()
+        index_items = List[Node]()
         commas = List[Token]()
 
-        is_paren = self.current_str() == '('
-        if is_paren:
-            self.skip()
-
         while True:
-            v = self.parse_expression(precedence['in'])  # prevent parsing of for's 'in'
-            index.append(v)
+            v = self.parse_expression(precedence['in'], star_expr_allowed=True)  # prevent parsing of for-stmt's 'in'
+            index_items.append(v)
             if self.current_str() != ',':
                 commas.append(none)
                 break
             commas.append(self.skip())
 
-        if is_paren:
-            self.expect(')')
+        if len(index_items) == 1:
+            index = index_items[0]
+        else:
+            index = TupleExpr(index_items)
+            index.set_line(index_items[0].get_line())
+            self.set_repr(index, noderepr.TupleExprRepr(none, commas, none))
 
-        return index, commas
+        return index
 
     def parse_if_stmt(self) -> IfStmt:
         is_error = False
@@ -1020,7 +1082,7 @@ class Parser:
 
     # Parsing expressions
 
-    def parse_expression(self, prec: int = 0) -> Node:
+    def parse_expression(self, prec: int = 0, star_expr_allowed: bool = False) -> Node:
         """Parse a subexpression within a specific precedence context."""
         expr = Undefined  # type: Node
         t = self.current()  # Remember token for setting the line number.
@@ -1040,6 +1102,8 @@ class Parser:
             expr = self.parse_lambda_expr()
         elif s == '{':
             expr = self.parse_dict_or_set_expr()
+        elif s == '*' and star_expr_allowed:
+            expr = self.parse_star_expr()
         else:
             if isinstance(self.current(), Name):
                 # Name expression.
@@ -1054,6 +1118,10 @@ class Parser:
                 expr = self.parse_unicode_literal()
             elif isinstance(self.current(), FloatLit):
                 expr = self.parse_float_expr()
+            elif isinstance(self.current(), ComplexLit):
+                expr = self.parse_complex_expr()
+            elif isinstance(t, Keyword) and s == "yield":
+                expr = self.parse_yield_from_expr() # The expression yield from and yield to assign
             else:
                 # Invalid expression.
                 self.parse_error()
@@ -1137,12 +1205,21 @@ class Parser:
             expr = self.parse_empty_tuple_expr(lparen)  # type: Node
         else:
             # Parenthesised expression.
-            expr = self.parse_expression(0)
+            expr = self.parse_expression(0, star_expr_allowed=True)
             rparen = self.expect(')')
             expr = ParenExpr(expr)
             self.set_repr(expr, noderepr.ParenExprRepr(lparen, rparen))
         return expr
 
+    def parse_star_expr(self) -> Node:
+        star = self.expect('*')
+        expr = self.parse_expression(precedence['*u'])
+        expr = StarExpr(expr)
+        if expr.line < 0:
+            expr.set_line(star)
+        self.set_repr(expr, noderepr.StarExprRepr(star))
+        return expr
+
     def parse_empty_tuple_expr(self, lparen: Any) -> TupleExpr:
         rparen = self.expect(')')
         node = TupleExpr([])
@@ -1155,7 +1232,7 @@ class Parser:
         lbracket = self.expect('[')
         commas = List[Token]()
         while self.current_str() != ']' and not self.eol():
-            items.append(self.parse_expression(precedence['<for>']))
+            items.append(self.parse_expression(precedence['<for>'], star_expr_allowed=True))
             if self.current_str() != ',':
                 break
             commas.append(self.expect(','))
@@ -1174,7 +1251,16 @@ class Parser:
             return expr
 
     def parse_generator_expr(self, left_expr: Node) -> GeneratorExpr:
-        indices = List[List[Node]]()
+        indices, sequences, condlists, for_toks, in_toks, if_toklists = self.parse_comp_for()
+
+        gen = GeneratorExpr(left_expr, indices, sequences, condlists)
+        gen.set_line(for_toks[0])
+        self.set_repr(gen, noderepr.GeneratorExprRepr(for_toks, in_toks, if_toklists))
+        return gen
+
+    def parse_comp_for(self) -> Tuple[List[Node], List[Node], List[List[Node]],
+                                      List[Token], List[Token], List[List[Token]]]:
+        indices = List[Node]()
         sequences = List[Node]()
         for_toks = List[Token]()
         in_toks = List[Token]()
@@ -1184,7 +1270,7 @@ class Parser:
             if_toks = List[Token]()
             conds = List[Node]()
             for_toks.append(self.expect('for'))
-            index, commas = self.parse_for_index_variables()
+            index = self.parse_for_index_variables()
             indices.append(index)
             in_toks.append(self.expect('in'))
             sequence = self.parse_expression_list()
@@ -1195,11 +1281,7 @@ class Parser:
             if_toklists.append(if_toks)
             condlists.append(conds)
 
-        gen = GeneratorExpr(left_expr, indices, sequences, condlists)
-        gen.set_line(for_toks[0])
-        self.set_repr(gen, noderepr.GeneratorExprRepr(for_toks, commas, in_toks,
-                                                      if_toklists))
-        return gen
+        return indices, sequences, condlists, for_toks, in_toks, if_toklists
 
     def parse_expression_list(self) -> Node:
         prec = precedence['<if>']
@@ -1223,13 +1305,17 @@ class Parser:
         colons = List[Token]()
         commas = List[Token]()
         while self.current_str() != '}' and not self.eol():
-            key = self.parse_expression(precedence[','])
+            key = self.parse_expression(precedence['<if>'])
             if self.current_str() in [',', '}'] and items == []:
                 return self.parse_set_expr(key, lbrace)
+            elif self.current_str() == 'for' and items == []:
+                return self.parse_set_comprehension(key, lbrace)
             elif self.current_str() != ':':
                 self.parse_error()
             colons.append(self.expect(':'))
-            value = self.parse_expression(precedence[','])
+            value = self.parse_expression(precedence['<if>'])
+            if self.current_str() == 'for' and items == []:
+                return self.parse_dict_comprehension(key, value, lbrace, colons[0])
             items.append((key, value))
             if self.current_str() != ',':
                 break
@@ -1254,6 +1340,21 @@ class Parser:
                                                      rbrace, none, none))
         return expr
 
+    def parse_set_comprehension(self, expr: Node, lbrace: Token):
+        gen = self.parse_generator_expr(expr)
+        rbrace = self.expect('}')
+        set_comp = SetComprehension(gen)
+        self.set_repr(set_comp, noderepr.SetComprehensionRepr(lbrace, rbrace))
+        return set_comp
+
+    def parse_dict_comprehension(self, key: Node, value: Node, lbrace: Token, colon: Token) -> DictionaryComprehension:
+        indices, sequences, condlists, for_toks, in_toks, if_toklists = self.parse_comp_for()
+        dic = DictionaryComprehension(key, value, indices, sequences, condlists)
+        dic.set_line(colon)
+        rbrace = self.expect('}')
+        self.set_repr(dic, noderepr.DictionaryComprehensionRepr(lbrace, colon, for_toks, in_toks, if_toklists, rbrace))
+        return dic
+
     def parse_tuple_expr(self, expr: Node,
                          prec: int = precedence[',']) -> TupleExpr:
         items = [expr]
@@ -1263,7 +1364,7 @@ class Parser:
             if (self.current_str() in [')', ']', '='] or
                     isinstance(self.current(), Break)):
                 break
-            items.append(self.parse_expression(prec))
+            items.append(self.parse_expression(prec, star_expr_allowed=True))
             if self.current_str() != ',': break
         node = TupleExpr(items)
         self.set_repr(node, noderepr.TupleExprRepr(none, commas, none))
@@ -1343,6 +1444,12 @@ class Parser:
         self.set_repr(node, noderepr.FloatExprRepr(tok))
         return node
 
+    def parse_complex_expr(self) -> ComplexExpr:
+        tok = self.expect_type(ComplexLit)
+        node = ComplexExpr(complex(tok.string))
+        self.set_repr(node, noderepr.ComplexExprRepr(tok))
+        return node
+
     def parse_call_expr(self, callee: Any) -> CallExpr:
         lparen = self.expect('(')
         (args, kinds, names,
@@ -1708,7 +1815,7 @@ def token_repr(tok: Token) -> str:
         return 'end of file'
     elif isinstance(tok, Keyword) or isinstance(tok, Name):
         return '"{}"'.format(tok.string)
-    elif isinstance(tok, IntLit) or isinstance(tok, FloatLit):
+    elif isinstance(tok, IntLit) or isinstance(tok, FloatLit) or isinstance(tok, ComplexLit):
         return 'numeric literal'
     elif isinstance(tok, StrLit):
         return 'string literal'
