@@ -16,7 +16,9 @@ from mypy.nodes import (
     UnicodeExpr, FloatExpr, CallExpr, SuperExpr, MemberExpr, IndexExpr,
     SliceExpr, OpExpr, UnaryExpr, FuncExpr, TypeApplication, PrintStmt,
     SymbolTable, RefExpr, UndefinedExpr, TypeVarExpr, DucktypeExpr,
-    DisjointclassExpr, ComparisonExpr, TempNode
+    DisjointclassExpr, ComparisonExpr, TempNode, StarExpr, YieldFromStmt,
+    YieldFromExpr, NamedTupleExpr, NonlocalDecl, SetComprehension,
+    DictionaryComprehension, ComplexExpr
 )
 from mypy.types import Type, FunctionLike
 from mypy.visitor import NodeVisitor
@@ -59,10 +61,10 @@ class TransformVisitor(NodeVisitor[Node]):
         return Import(node.ids[:])
 
     def visit_import_from(self, node: ImportFrom) -> Node:
-        return ImportFrom(node.id, node.names[:])
+        return ImportFrom(node.id, node.relative, node.names[:])
 
     def visit_import_all(self, node: ImportAll) -> Node:
-        return ImportAll(node.id)
+        return ImportAll(node.id, node.relative)
 
     def visit_func_def(self, node: FuncDef) -> FuncDef:
         # Note that a FuncDef must be transformed to a FuncDef.
@@ -148,6 +150,9 @@ class TransformVisitor(NodeVisitor[Node]):
     def visit_global_decl(self, node: GlobalDecl) -> Node:
         return GlobalDecl(node.names[:])
 
+    def visit_nonlocal_decl(self, node: NonlocalDecl) -> Node:
+        return NonlocalDecl(node.names[:])
+
     def visit_block(self, node: Block) -> Block:
         return Block(self.nodes(node.body))
 
@@ -203,7 +208,7 @@ class TransformVisitor(NodeVisitor[Node]):
                          self.optional_block(node.else_body))
 
     def visit_for_stmt(self, node: ForStmt) -> Node:
-        return ForStmt(self.nodes(node.index),
+        return ForStmt(self.node(node.index),
                        self.node(node.expr),
                        self.block(node.body),
                        self.optional_block(node.else_body))
@@ -217,6 +222,9 @@ class TransformVisitor(NodeVisitor[Node]):
     def visit_yield_stmt(self, node: YieldStmt) -> Node:
         return YieldStmt(self.node(node.expr))
 
+    def visit_yield_from_stmt(self, node: YieldFromStmt) -> Node:
+        return YieldFromStmt(self.node(node.expr))
+
     def visit_del_stmt(self, node: DelStmt) -> Node:
         return DelStmt(self.node(node.expr))
 
@@ -255,6 +263,9 @@ class TransformVisitor(NodeVisitor[Node]):
         return PrintStmt(self.nodes(node.args),
                          node.newline)
 
+    def visit_star_expr(self, node: StarExpr) -> Node:
+        return StarExpr(node.expr)
+
     def visit_int_expr(self, node: IntExpr) -> Node:
         return IntExpr(node.value)
 
@@ -270,6 +281,9 @@ class TransformVisitor(NodeVisitor[Node]):
     def visit_float_expr(self, node: FloatExpr) -> Node:
         return FloatExpr(node.value)
 
+    def visit_complex_expr(self, node: ComplexExpr) -> Node:
+        return ComplexExpr(node.value)
+
     def visit_paren_expr(self, node: ParenExpr) -> Node:
         return ParenExpr(self.node(node.expr))
 
@@ -301,6 +315,9 @@ class TransformVisitor(NodeVisitor[Node]):
         new.node = target
         new.is_def = original.is_def
 
+    def visit_yield_from_expr(self, node: YieldFromExpr) -> Node:
+        return YieldFromExpr(self.node(node.expr))
+
     def visit_call_expr(self, node: CallExpr) -> Node:
         return CallExpr(self.node(node.callee),
                         self.nodes(node.args),
@@ -366,12 +383,24 @@ class TransformVisitor(NodeVisitor[Node]):
         generator.set_line(node.generator.line)
         return ListComprehension(generator)
 
+    def visit_set_comprehension(self, node: SetComprehension) -> Node:
+        generator = self.duplicate_generator(node.generator)
+        generator.set_line(node.generator.line)
+        return SetComprehension(generator)
+
+    def visit_dictionary_comprehension(self, node: DictionaryComprehension) -> Node:
+        return DictionaryComprehension(self.node(node.key), self.node(node.value),
+                             [self.node(index) for index in node.indices],
+                             [self.node(s) for s in node.sequences],
+                             [[self.node(cond) for cond in conditions]
+                              for conditions in node.condlists])
+
     def visit_generator_expr(self, node: GeneratorExpr) -> Node:
         return self.duplicate_generator(node)
 
     def duplicate_generator(self, node: GeneratorExpr) -> GeneratorExpr:
         return GeneratorExpr(self.node(node.left_expr),
-                             [self.nodes(index) for index in node.indices],
+                             [self.node(index) for index in node.indices],
                              [self.node(s) for s in node.sequences],
                              [[self.node(cond) for cond in conditions]
                               for conditions in node.condlists])
@@ -390,6 +419,9 @@ class TransformVisitor(NodeVisitor[Node]):
         return TypeVarExpr(node.name(), node.fullname(),
                            self.types(node.values))
 
+    def visit_namedtuple_expr(self, node: NamedTupleExpr) -> Node:
+        return NamedTupleExpr(node.info)
+
     def visit_ducktype_expr(self, node: DucktypeExpr) -> Node:
         return DucktypeExpr(node.type)
 
