@@ -285,7 +285,10 @@ class Callable(FunctionLike):
 
     def type_object(self) -> mypy.nodes.TypeInfo:
         assert self.is_type_obj()
-        return cast(Instance, self.ret_type).type
+        ret = self.ret_type
+        if isinstance(ret, TupleType):
+            ret = ret.fallback
+        return cast(Instance, ret).type
 
     def accept(self, visitor: 'TypeVisitor[T]') -> T:
         return visitor.visit_callable(self)
@@ -390,6 +393,19 @@ class TupleType(Type):
         return visitor.visit_tuple_type(self)
 
 
+class StarType(Type):
+    """The star type *type_parameter"""
+
+    type = Undefined(Type)
+
+    def __init__(self, type: Type, line: int = -1, repr: Any = None) -> None:
+        self.type = type
+        super().__init__(line, repr)
+
+    def accept(self, visitor: 'TypeVisitor[T]') -> T:
+        return visitor.visit_star_type(self)
+
+
 class UnionType(Type):
     """The union type Union[T1, ..., Tn] (at least one type argument)."""
 
@@ -512,6 +528,9 @@ class TypeVisitor(Generic[T]):
     def visit_tuple_type(self, t: TupleType) -> T:
         pass
 
+    def visit_star_type(self, t: StarType) -> T:
+        pass
+
     def visit_union_type(self, t: UnionType) -> T:
         assert(0)               # XXX catch visitors that don't have Union cases yet
 
@@ -569,6 +588,9 @@ class TypeTranslator(TypeVisitor[Type]):
                          Any(t.fallback.accept(self)),
                          t.line, t.repr)
 
+    def visit_star_type(self, t: StarType) -> Type:
+        return StarType(t.type.accept(self), t.line, t.repr)
+
     def visit_union_type(self, t: UnionType) -> Type:
         return UnionType(self.translate_types(t.items), t.line, t.repr)
 
@@ -587,6 +609,8 @@ class TypeTranslator(TypeVisitor[Type]):
 class TypeStrVisitor(TypeVisitor[str]):
     """Visitor for pretty-printing types into strings.
 
+    This is mostly for debugging/testing.
+
     Do not preserve original formatting.
 
     Notes:
@@ -680,8 +704,16 @@ class TypeStrVisitor(TypeVisitor[str]):
 
     def visit_tuple_type(self, t):
         s = self.list_str(t.items)
+        if t.fallback:
+            fallback_name = t.fallback.type.fullname()
+            if fallback_name != 'builtins.tuple':
+                return 'Tuple[{}, fallback={}]'.format(s, t.fallback.accept(self))
         return 'Tuple[{}]'.format(s)
 
+    def visit_star_type(self, t):
+        s = t.type.accept(self)
+        return '*{}'.format(s)
+
     def visit_union_type(self, t):
         s = self.list_str(t.items)
         return 'Union[{}]'.format(s)
@@ -763,6 +795,9 @@ class TypeQuery(TypeVisitor[bool]):
     def visit_tuple_type(self, t: TupleType) -> bool:
         return self.query_types(t.items)
 
+    def visit_star_type(self, t: StarType) -> bool:
+        return t.type.accept(self)
+
     def visit_union_type(self, t: UnionType) -> bool:
         return self.query_types(t.items)
 
